
local autoplace_utils = require("autoplace_utils")
local utils = require('utils')

function main()
  define_crystals()
  define_lava_cracks()
  define_lava_furnace()
end

--------------------------------------------------------------------------------
-- This defines quartz and woflramite crystals. They work as trees.

function define_crystals()
  for suffix, params in pairs({
    ['-quartz']     = { num_pictures = 5,
                        map_color = { r = 0.35, g = 0.35, b = 0.35 },
                        max_probability = 0.8
                      },
    ['-wolframite'] = { num_pictures = 5,
                        map_color = { r = 0.45, g = 0.35, b = 0.25 },
                        max_probability = 0.3
                      }
  }) do
    define_crystal(suffix, params.num_pictures,
                   params.map_color, params.max_probability)
  end
end

function define_crystal(suffix, num_pictures, map_color, max_probability)
  local crystal_name = string.format('crystal%s', suffix)

  local pictures = {}
  for picture = 1, num_pictures do
    pictures[picture] = {
      filename = string.format(
        '__%s__/graphics/entity/tree/%s/%s-%02d.png',
        utils.mod_name, crystal_name, crystal_name, picture
      ),
      flags = { 'mipmap' },
      width = 75,
      height = 60
    }
  end

  data.raw['tree'][crystal_name] = {
    type = 'tree',
    name = crystal_name,
    icon = string.format('__%s__/graphics/icons/%s.png',
                         utils.mod_name, crystal_name),
    flags = { 'placeable-neutral', 'placeable-off-grid' },
    minable = {
      mining_particle = 'stone-particle',
      mining_time = 10,
      result = 'stone',
      count = 4
    },
    mined_sound = { filename = '__base__/sound/deconstruct-bricks.ogg' },
    max_health = 100,
    collision_box = {{ -0.4, -0.6 }, { 0.4, 0.2 }},
    selection_box = {{ -0.4, -0.6 }, { 0.4, 0.2 }},
    drawing_box   = {{ -0.4, -0.6 }, { 0.4, 0.2 }},
    subgroup = 'trees',
    order = string.format('a[tree]-c[%s]', crystal_name),
    vehicle_impact_sound = { filename = '__base__/sound/car-metal-impact.ogg' },
    light = { minimum_darkness = 0.1, intensity = 1, size = 50 },
    autoplace = {
      max_probability = max_probability,
      sharpness = 1.0,
      coverage = 0.05,
      order = 'a[tree]-b[forest]',
      peaks = autoplace_utils.peaks({{{ 35, 0.3}, {-5, 0}, -0.3}},
                                    {
                                      {
                                        influence = 0.55
                                      },
                                      {
                                        noise_layer = 'trees',
                                        noise_persistence = 0.92,
                                        noise_octaves_difference = -1.2
                                      }
                                    }),
    },
    pictures = pictures,
    map_color = map_color
  }
end


--------------------------------------------------------------------------------
-- This defines lava cracks covering the ground. They work as grass.

function define_lava_cracks()
  data:extend({
    {
      type = 'decorative',
      name = 'lava-crack',
      flags = { 'placeable-neutral', 'placeable-off-grid', 'not-on-map' },
      icon = string.format(
          '__%s__/graphics/icons/lava-crack.png',
          utils.mod_name
      ),
      subgroup = 'grass',
      order = 'b[decorative]-e[cane]-b[single]-a[brown]',
      collision_box = {{ -2, -2 }, { 2, 2 }},
      selection_box = {{ -1, -1 }, { 1, 1 }},
      render_layer = "object",
      selectable_in_game = false,
      autoplace = {
        order = "a[doodad]-f[grass]-b",
        sharpness = 0.2,
        max_probability = 0.1,
        peaks = autoplace_utils.peaks(grass_peaks,
                                      {
                                        {
                                          influence = 0.3,
                                          noise_layer = "grass2",
                                          noise_octaves_difference = -2.8,
                                          noise_persistence = 0.5
                                        },
                                      }),
      },
      pictures = {
        {
          filename = string.format(
              '__%s__/graphics/entity/decorative/lava-crack/lava-crack-01.png',
              utils.mod_name
          ),
          width = 68,
          height = 44,
          shift = { 0.0, 2.0 }
        },
        {
          filename = string.format(
              '__%s__/graphics/entity/decorative/lava-crack/lava-crack-02.png',
              utils.mod_name
          ),
          width = 80,
          height = 71,
          shift = { 0.0, 2.0 }
        },
        {
          filename = string.format(
              '__%s__/graphics/entity/decorative/lava-crack/lava-crack-03.png',
              utils.mod_name
          ),
          width = 56,
          height = 105,
          shift = { 0.0, 2.0 }
        },
        {
          filename = string.format(
              '__%s__/graphics/entity/decorative/lava-crack/lava-crack-04.png',
              utils.mod_name
          ),
          width = 105,
          height = 56,
          shift = { 0.0, 2.0 }
        },
        {
          filename = string.format(
              '__%s__/graphics/entity/decorative/lava-crack/lava-crack-05.png',
              utils.mod_name
          ),
          width = 68,
          height = 44,
          shift = { 0.0, 2.0 }
        },
        {
          filename = string.format(
              '__%s__/graphics/entity/decorative/lava-crack/lava-crack-06.png',
              utils.mod_name
          ),
          width = 80,
          height = 71,
          shift = { 0.0, 2.0 }
        },
        {
          filename = string.format(
              '__%s__/graphics/entity/decorative/lava-crack/lava-crack-07.png',
              utils.mod_name
          ),
          width = 60,
          height = 60,
          shift = { 0.0, 2.0 }
        },
      }
    }
  })
end

--------------------------------------------------------------------------------
-- This defines the lava furnace.
-- HACK: Lava furnace is a furnace that gets placed near lava, like offshore
-- pump near water. Because it's impossible to restrict placement of a furnace,
-- we define a copy of the furnace with pump type, and in control.lua we replace
-- it with the actual furnace upon building.
-- HACK: Moreover, because furnaces don't support rotation, we make four
-- variants of the actual furnace entity, one for each direction (north, east,
-- south, west), and pick the one that currespond to the pump's direction when
-- replacing the pump.
-- HACK: Because offshore pump normally has a pipe connection, but we don't want
-- to see those when placing the lava furnace, we set their picture to
-- an empty (100% transparent) png.

function define_lava_furnace()
  local function nopipecoverspictures()
    return {
      north =
      {
        filename = string.format("__%s__/graphics/empty.png", utils.mod_name),
        priority = "extra-high",
        width = 1,
        height = 1
      },
      east =
      {
        filename = string.format("__%s__/graphics/empty.png", utils.mod_name),
        priority = "extra-high",
        width = 1,
        height = 1
      },
      south =
      {
        filename = string.format("__%s__/graphics/empty.png", utils.mod_name),
        priority = "extra-high",
        width = 1,
        height = 1
      },
      west =
      {
        filename = string.format("__%s__/graphics/empty.png", utils.mod_name),
        priority = "extra-high",
        width = 1,
        height = 1
      }
    }
  end

  local hack_lava_furnace_common = {
    order = 'd-a-a', -- for some reason this is required for entities
                     -- that can't be placed with items.
    icon = string.format("__%s__/graphics/icons/lava-furnace.png",
                         utils.mod_name),
    flags = { 'placeable-neutral', 'player-creation', 'filter-directions' },
    minable = { mining_time = 1, result = 'lava-furnace' },
    max_health = 80,
    corpse = "small-remnants",
    resistances =
    {
      {
        type = "fire",
        percent = 70
      }
    },
    tile_width = 1,
    tile_height = 1,
    vehicle_impact_sound =  {
        filename = "__base__/sound/car-metal-impact.ogg",
        volume = 0.65
    },
    working_sound = {
      sound = { filename = '__base__/sound/furnace.ogg' },
    },
    crafting_categories = { 'smelting' },
    crafting_speed = 1,
    energy_usage = '1W',
    ingredient_count = 1,
    energy_source = {
      type = 'burner',
      effectivity = 1,
      emissions = 0.2,
      fuel_inventory_size = 0,
    },
    result_inventory_size = 1,
    source_inventory_size = 1,
    circuit_wire_connection_points =
    {
      {
        shadow =
        {
          red = {2.71875, 0.375},
          green = {2.5, 0.375},
        },
        wire =
        {
          red = {0.84375, -0.09375},
          green = {0.6875, -0.09375},
        }
      },
      {
        shadow =
        {
          red = {0.765625, 0.546875},
          green = {0.765625, 0.421875},
        },
        wire =
        {
          red = {-0.28125, -0.09375},
          green = {-0.28125, -0.21875},
        }
      },
      {
        shadow =
        {
          red = {-0.09375, 0.5625},
          green = {0.0625, 0.5625},
        },
        wire =
        {
          red = {-0.90625, -0.53125},
          green = {-0.75, -0.53125},
        }
      },
      {
        shadow =
        {
          red = {1.78125, -0.46875},
          green = {1.78125, -0.3125},
        },
        wire =
        {
          red = {0.34375, -1.40625},
          green = {0.34375, -1.25},
        }
      }
    },
    circuit_connector_sprites =
    {
      get_circuit_connector_sprites({0.90625, -0.15625}, nil, 0),
      get_circuit_connector_sprites({0, 0.03125}, nil, 6),
      get_circuit_connector_sprites({-0.9375, -0.25}, nil, 4),
      get_circuit_connector_sprites({0.125, -1.3125}, nil, 2),
    },
    circuit_wire_max_distance = 7.5
  }

  data:extend({
    utils.merge_table(hack_lava_furnace_common, {
      type = 'offshore-pump',
      name = 'hack-lava-furnace-as-pump',
      collision_box = {{-0.4, -0.1}, {0.4, 0.7}},
      selection_box = {{-1, -0.1}, {1, 1.9}},
      fluid = "water",
      pumping_speed = 1,
      fluid_box =
      {
        base_area = 1,
        pipe_covers = nopipecoverspictures(),
        pipe_connections =
        {
          { position = {0, 1} },
        },
      },
      picture = {
        north = {
          filename = string.format(
              "__%s__/graphics/entity/offshore-pump/lava-furnace.png",
              utils.mod_name
          ),
          priority = "high",
          shift = { 0.9, 0.4 },
          width = 160,
          height = 102
        },
        east = {
          filename = string.format(
              "__%s__/graphics/entity/offshore-pump/lava-furnace.png",
              utils.mod_name
          ),
          priority = "high",
          shift = { 0.3, 0.0 },
          x = 160,
          width = 160,
          height = 102
        },
        south = {
          filename = string.format(
              "__%s__/graphics/entity/offshore-pump/lava-furnace.png",
              utils.mod_name
          ),
          priority = "high",
          shift = { 0.9, -0.7 },
          x = 320,
          width = 160,
          height = 102
        },
        west = {
          filename = string.format(
              "__%s__/graphics/entity/offshore-pump/lava-furnace.png",
              utils.mod_name
          ),
          priority = "high",
          shift = { 1.6, 0.0 },
          x = 480,
          width = 160,
          height = 102
        }
      },
    }),
    utils.merge_table(hack_lava_furnace_common, {
      type = 'furnace',
      name = "hack-lava-furnace-north",
      collision_box = {{ -0.9, -0.1 }, { 0.9, 1.1 }},
      selection_box = {{ -1.0, -0.1 }, { 1.0, 1.9 }},
      animation = {
        filename = string.format(
            "__%s__/graphics/entity/offshore-pump/lava-furnace.png",
            utils.mod_name
        ),
        priority = "high",
        shift = { 0.9, 0.4 },
        frame_count = 1,
        width = 160,
        height = 102
      },
      working_visualisations = {
        {
          animation = {
            filename = string.format(
                "__%s__/graphics/entity/offshore-pump/lava-furnace-fire.png",
                utils.mod_name
            ),
            shift = { 0.9, 0.4 },
            frame_count = 1,
            width = 160,
            height = 102
          },
        }
      }
    }),
    utils.merge_table(hack_lava_furnace_common, {
      type = 'furnace',
      name = "hack-lava-furnace-east",
      collision_box = {{ -1.1, -0.9 }, { 0.1, 0.9 }},
      selection_box = {{ -1.9, -1.0 }, { 0.1, 1.0 }},
      animation = {
        filename = string.format(
            "__%s__/graphics/entity/offshore-pump/lava-furnace.png",
            utils.mod_name
        ),
        priority = "high",
        shift = { 0.3, 0.0 },
        frame_count = 1,
        x = 160,
        width = 160,
        height = 102
      },
      working_visualisations = {
        {
          animation = {
            filename = string.format(
                "__%s__/graphics/entity/offshore-pump/lava-furnace-fire.png",
                utils.mod_name
            ),
            shift = { 0.3, 0.0 },
            frame_count = 1,
            x = 160,
            width = 160,
            height = 102
          },
        }
      }
    }),
    utils.merge_table(hack_lava_furnace_common, {
      type = 'furnace',
      name = "hack-lava-furnace-south",
      collision_box = {{ -0.9, -1.1 }, { 0.9, 0.1 }},
      selection_box = {{ -1.0, -1.9 }, { 1.0, 0.1 }},
      animation = {
        filename = string.format(
            "__%s__/graphics/entity/offshore-pump/lava-furnace.png",
            utils.mod_name
        ),
        priority = "high",
        shift = { 0.9, -0.7 },
         frame_count = 1,
        x = 320,
        width = 160,
        height = 102
      },
      working_visualisations = {
        {
          animation = {
            filename = string.format(
                "__%s__/graphics/entity/offshore-pump/lava-furnace-fire.png",
                utils.mod_name
            ),
            shift = { 0.9, -0.7 },
            frame_count = 1,
            x = 320,
            width = 160,
            height = 102
          },
        }
      }
    }),
    utils.merge_table(hack_lava_furnace_common, {
      type = 'furnace',
      name = "hack-lava-furnace-west",
      collision_box = {{ -0.1, -0.9 }, { 1.1, 0.9 }},
      selection_box = {{ -0.1, -1.0 }, { 1.9, 1.0 }},
      animation = {
        filename = string.format(
            "__%s__/graphics/entity/offshore-pump/lava-furnace.png",
            utils.mod_name),
        priority = "high",
        shift = { 1.6, 0.0 },
        frame_count = 1,
        x = 480,
        width = 160,
        height = 102
      },
      working_visualisations = {
        {
          animation = {
            filename = string.format(
                "__%s__/graphics/entity/offshore-pump/lava-furnace-fire.png",
                utils.mod_name),
            shift = { 1.6, 0.0 },
            frame_count = 1,
            x = 480,
            width = 160,
            height = 102
          },
          smoke = {
            {
              name = "smoke",
              frequency = 10,
              position = {0.7, -1.2},
              starting_vertical_speed = 0.08,
              starting_frame_deviation = 60
            }
          }
        }
      }
    }),
    {
      type = "item",
      name = "lava-furnace",
      icon = string.format("__%s__/graphics/icons/lava-furnace.png",
                           utils.mod_name),
      flags = {"goes-to-quickbar"},
      subgroup = "extraction-machine",
      order = "a[lava-furnace]",
      place_result = "hack-lava-furnace-as-pump",
      stack_size = 20
    },
  })
end

--------------------------------------------------------------------------------
-- Finally do something.

main()
